@@ -6,6 +6,8 @@
 
 import pytest
 import os
+import h5py
+import itertools
 import numpy as np
 
 from pyuvdata import UVData
@@ -29,6 +31,8 @@ testfiles = [
     "1061315448_20130823175130_mini_gpubox07_01.fits",
     "1061315448.metafits",
     "1061315448_20130823175130_mini_vv_07_01.uvh5",
+    "1320409688.metafits",
+    "1320409688_20211108122750_mini_ch137_000.fits",
 ]
 filelist = [os.path.join(testdir, filei) for filei in testfiles]
 
@@ -76,7 +80,6 @@ def test_read_mwa_write_uvfits(tmp_path):
     mwa_uv = UVData()
     uvfits_uv = UVData()
     messages = [
-        "telescope_location is not set",
         "some coarse channel files were not submitted",
     ]
     with uvtest.check_warnings(UserWarning, messages):
@@ -100,8 +103,66 @@ def test_read_mwa_write_uvfits(tmp_path):
     assert mwa_uv == uvfits_uv
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set. ")
+@pytest.mark.filterwarnings("ignore:antnums_to_baseline")
+@pytest.mark.filterwarnings("ignore:Found antenna numbers > 256 in this data")
+def test_read_mwax_write_uvfits(tmp_path):
+    """
+    MWAX correlator fits to uvfits loopback test.
+
+    Read in MWAX correlator files, write out as uvfits, read back in and check
+    for object equality.
+    """
+    # spoof testfile to contain 2 times and 2 freqs
+    spoof_file = str(tmp_path / "mwax_spoof_ch137_000.fits")
+    with fits.open(filelist[12]) as mini1:
+        mini1[0].header["NFINECHS"] = 2
+        mini1[1].data = np.repeat(mini1[1].data, 2, axis=1)
+        mini1[2].data = np.repeat(mini1[2].data, 2, axis=1)
+        extra_dat = np.copy(mini1[1].data)
+        extra_samps = np.copy(mini1[2].data)
+        # mini1.append(fits.HDUList([mini1[1], mini1[2]]))
+        mini1.append(fits.ImageHDU(extra_dat))
+        mini1.append(fits.ImageHDU(extra_samps))
+        mini1[3].header["TIME"] = 1636374472
+        mini1[4].header["TIME"] = 1636374472
+        mini1[3].header["MILLITIM"] = 0
+        mini1[4].header["MILLITIM"] = 0
+        mini1.writeto(spoof_file)
+    mwax_uv = UVData()
+    uvfits_uv = UVData()
+    messages = [
+        "Fixing auto-correlations to be be real-only, after some imaginary values",
+        "some coarse channel files were not submitted",
+    ]
+    with uvtest.check_warnings(UserWarning, messages):
+        mwax_uv.read_mwa_corr_fits(
+            [spoof_file, filelist[11]],
+            correct_cable_len=True,
+            phase_to_pointing_center=True,
+        )
+    testfile = str(tmp_path / "outtest_MWAXcorr.uvfits")
+    mwax_uv.write_uvfits(testfile, spoof_nonessential=True)
+    uvfits_uv.read_uvfits(testfile)
+
+    assert mwax_uv == uvfits_uv
+
+
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
+def test_mwax_metafits_keys(tmp_path):
+    """Check that mwax keywords are removed from extra_keywords for legacy files"""
+    meta_spoof_file = str(tmp_path / "spoof_1131733552.metafits")
+    with fits.open(filelist[0]) as meta:
+        meta[0].header["DELAYMOD"] = None
+        meta.writeto(meta_spoof_file)
+    uv = UVData()
+    uv.read([meta_spoof_file, filelist[1]])
+
+    assert "DELAYMOD" not in uv.extra_keywords.keys()
+
+
+@pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
 def test_select_on_read():
     mwa_uv = UVData()
     mwa_uv2 = UVData()
@@ -118,7 +179,6 @@ def test_select_on_read():
         UserWarning,
         [
             'Warning: select on read keyword set, but file_type is "mwa_corr_fits"',
-            "telescope_location is not set. Using known values for MWA.",
             "some coarse channel files were not submitted",
         ],
     ):
@@ -130,7 +190,6 @@ def test_select_on_read():
     assert mwa_uv == mwa_uv2
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set. ")
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
 def test_read_mwa_read_cotter():
     """
@@ -162,6 +221,7 @@ def test_read_mwa_read_cotter():
         atol=1e-4,
         rtol=0,
     )
+    assert mwa_uv.freq_array == cotter_uv.freq_array
 
 
 def test_read_mwa_write_uvfits_meta_mod(tmp_path):
@@ -176,7 +236,6 @@ def test_read_mwa_write_uvfits_meta_mod(tmp_path):
     mwa_uv = UVData()
     uvfits_uv = UVData()
     messages = [
-        "telescope_location is not set",
         "some coarse channel files were not submitted",
     ]
     files = [filelist[1], filelist[5]]
@@ -198,9 +257,9 @@ def test_read_mwa_write_uvfits_meta_mod(tmp_path):
     assert mwa_uv == uvfits_uv
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set. ")
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
 @pytest.mark.filterwarnings("ignore:Combined frequencies are separated by more than")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
 def test_read_mwa_multi():
     """Test reading in two sets of files."""
     set1 = filelist[0:2]
@@ -211,10 +270,10 @@ def test_read_mwa_multi():
 
     mwa_uv2 = UVData()
     messages = [
-        "telescope_location is not set",
         "some coarse channel files were not submitted",
-        "telescope_location is not set",
+        "cable length correction is now defaulted to True",
         "some coarse channel files were not submitted",
+        "cable length correction is now defaulted to True",
         "Combined frequencies are separated by more than their channel width",
     ]
     with uvtest.check_warnings(UserWarning, messages):
@@ -223,9 +282,9 @@ def test_read_mwa_multi():
     assert mwa_uv == mwa_uv2
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set. ")
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
 @pytest.mark.filterwarnings("ignore:Combined frequencies are separated by more than")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
 def test_read_mwa_multi_concat(tmp_path):
     """Test reading in two sets of files with fast concatenation."""
     # modify file so that time arrays are matching
@@ -240,10 +299,10 @@ def test_read_mwa_multi_concat(tmp_path):
 
     mwa_uv2 = UVData()
     messages = [
-        "telescope_location is not set",
         "some coarse channel files were not submitted",
-        "telescope_location is not set",
+        "cable length correction is now defaulted to True",
         "some coarse channel files were not submitted",
+        "cable length correction is now defaulted to True",
         "Combined frequencies are separated by more than their channel width",
     ]
     with uvtest.check_warnings(UserWarning, messages):
@@ -253,25 +312,24 @@ def test_read_mwa_multi_concat(tmp_path):
     os.remove(mod_mini_6)
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set. ")
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
 def test_read_mwa_flags():
     """Test handling of flag files."""
     mwa_uv = UVData()
     subfiles = [filelist[0], filelist[1], filelist[3], filelist[4]]
     messages = [
-        "mwaf files submitted with use_cotter_flags=False",
-        "telescope_location is not set",
+        "mwaf files submitted with use_aoflagger_flags=False",
         "some coarse channel files were not submitted",
+        "cable length correction is now defaulted to True",
     ]
     with uvtest.check_warnings(UserWarning, messages):
-        mwa_uv.read(subfiles, use_cotter_flags=False)
+        mwa_uv.read(subfiles, use_aoflagger_flags=False)
 
     del mwa_uv
 
     mwa_uv = UVData()
     with pytest.raises(ValueError) as cm:
-        mwa_uv.read(subfiles[0:2], use_cotter_flags=True)
+        mwa_uv.read(subfiles[0:2], use_aoflagger_flags=True)
     assert str(cm.value).startswith("no flag files submitted")
     del mwa_uv
 
@@ -288,9 +346,9 @@ def test_multiple_coarse():
     mwa_uv1 = UVData()
     mwa_uv2 = UVData()
     messages = [
-        "telescope_location is not set",
         "coarse channels are not contiguous for this observation",
         "some coarse channel files were not submitted",
+        "cable length correction is now defaulted to True",
     ]
     with uvtest.check_warnings(UserWarning, messages):
         mwa_uv1.read(order1)
@@ -300,8 +358,8 @@ def test_multiple_coarse():
     assert mwa_uv1 == mwa_uv2
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set. ")
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
 def test_ppds(tmp_path):
     """Test handling of ppds files"""
     # turnaround test with just ppds file given
@@ -348,10 +406,30 @@ def test_fine_channels(tmp_path):
         mini6.writeto(bad_fine)
     with pytest.raises(ValueError) as cm:
         mwa_uv.read([bad_fine, filelist[1]])
-    assert str(cm.value).startswith("files submitted have different fine")
+    assert str(cm.value).startswith("files submitted have different numbers")
     del mwa_uv
 
 
+def test_fine_channels_mwax(tmp_path):
+    """
+    Break read_mwa_corr_fits by submitting mwax files with different fine channels.
+
+    Test that error is raised if files with different numbers of fine channels
+    are submitted.
+    """
+    mwax_uv = UVData()
+    bad_fine = str(tmp_path / "bad_ch137_000.fits.fits")
+    with fits.open(filelist[12]) as mini:
+        mini[1].data = np.repeat(mini[1].data, 2, axis=1)
+        mini[2].data = np.repeat(mini[2].data, 2, axis=1)
+        mini[0].header["NFINECHS"] = 2
+        mini.writeto(bad_fine)
+    with pytest.raises(ValueError) as cm:
+        mwax_uv.read([bad_fine, filelist[12]])
+    assert str(cm.value).startswith("files submitted have different numbers")
+    del mwax_uv
+
+
 @pytest.mark.parametrize(
     "files,err_msg",
     [
@@ -425,11 +503,11 @@ def test_misaligned_times(tmp_path):
     del mwa_uv
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set. ")
 @pytest.mark.filterwarnings(
     "ignore:coarse channels are not contiguous for this observation"
 )
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
 def test_flag_nsample_basic():
     """
     Test that the flag(without flag_int) and nsample arrays correctly reflect data.
@@ -443,10 +521,8 @@ def test_flag_nsample_basic():
     )
     # check that only bad antennas are flagged for all times, freqs, pols
     bad_ants = [59, 114]
-    good_ants = list(range(128))
-    for j in bad_ants:
-        good_ants.remove(j)
-    bad = uv.select(antenna_nums=bad_ants, inplace=False)
+    good_ants = np.delete(np.unique(uv.ant_1_array), bad_ants)
+    bad = uv.select(antenna_nums=np.unique(uv.ant_1_array)[bad_ants], inplace=False)
     good = uv.select(antenna_nums=good_ants, inplace=False)
     assert np.all(bad.flag_array)
     good.flag_array = good.flag_array.reshape(
@@ -470,11 +546,11 @@ def test_flag_nsample_basic():
     assert np.all(uv.nsample_array[-1, :, 1, :] == 0.0)
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set. ")
 @pytest.mark.filterwarnings(
     "ignore:coarse channels are not contiguous for this observation"
 )
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
 def test_flag_init(flag_file_init):
     """
     Test that routine MWA flagging works as intended.
@@ -499,11 +575,11 @@ def test_flag_init(flag_file_init):
     ), "Some non-edge/center channels are entirely flagged!"
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set. ")
 @pytest.mark.filterwarnings(
     "ignore:coarse channels are not contiguous for this observation"
 )
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
 def test_flag_start_flag(flag_file_init):
     uv = UVData()
     uv.read(
@@ -548,11 +624,11 @@ def test_flag_start_flag(flag_file_init):
         ),
     ],
 )
-@pytest.mark.filterwarnings("ignore:telescope_location is not set. ")
 @pytest.mark.filterwarnings(
     "ignore:coarse channels are not contiguous for this observation"
 )
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
 def test_flag_init_errors(flag_file_init, err_type, read_kwargs, err_msg):
     uv = UVData()
     # give noninteger multiple inputs
@@ -565,7 +641,6 @@ def test_read_metadata_only(tmp_path):
     """Test reading an MWA corr fits file as metadata only."""
     uvd = UVData()
     messages = [
-        "telescope_location is not set",
         "some coarse channel files were not submitted",
     ]
     with uvtest.check_warnings(UserWarning, messages):
@@ -579,8 +654,8 @@ def test_read_metadata_only(tmp_path):
     assert uvd.metadata_only
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set.")
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
 def test_data_array_precision():
     uv = UVData()
     uv2 = UVData()
@@ -596,8 +671,8 @@ def test_data_array_precision():
     return
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set.")
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
 def test_nsample_array_precision():
     uv = UVData()
     uv2 = UVData()
@@ -631,73 +706,86 @@ def test_invalid_precision_errors():
     return
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set.")
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
 def test_remove_dig_gains():
     """Test digital gain removal."""
     uv1 = UVData()
-    uv1.read(filelist[0:2])
+    uv1.read(filelist[0:2], data_array_dtype=np.complex64)
 
     uv2 = UVData()
     uv2.read(filelist[0:2], remove_dig_gains=False)
 
     with fits.open(filelist[0]) as meta:
         meta_tbl = meta[1].data
-        antenna_numbers = meta_tbl["Antenna"][1::2]
+        antenna_inds = meta_tbl["Antenna"][1::2]
         dig_gains = meta_tbl["Gains"][1::2, :].astype(np.float64) / 64
-    reordered_inds = antenna_numbers.argsort()
-    dig_gains = dig_gains[reordered_inds, :]
-    dig_gains = dig_gains[:, np.array([23])]
-    dig_gains = np.repeat(dig_gains, 1, axis=1)
-    dig_gains1 = dig_gains[uv2.ant_1_array, :]
-    dig_gains2 = dig_gains[uv2.ant_2_array, :]
-    dig_gains1 = dig_gains1[:, :, np.newaxis, np.newaxis]
-    dig_gains2 = dig_gains2[:, :, np.newaxis, np.newaxis]
-    uv2.data_array = uv2.data_array / (dig_gains1 * dig_gains2)
+    reordered_inds = antenna_inds.argsort()
+    bad_ant_inds = [59, 114]
+    good_ants = np.delete(reordered_inds, bad_ant_inds)
+    ant_1_inds, ant_2_inds = np.transpose(
+        list(itertools.combinations_with_replacement(np.arange(uv2.Nants_data), 2))
+    )
+    ant_1_inds = np.tile(np.array(ant_1_inds), uv2.Ntimes).astype(np.int_)
+    ant_2_inds = np.tile(np.array(ant_2_inds), uv2.Ntimes).astype(np.int_)
+    dig_gains = dig_gains[good_ants, 23]
+    uv2.data_array = uv2.data_array / (
+        dig_gains[ant_1_inds, np.newaxis, np.newaxis, np.newaxis]
+        * dig_gains[ant_2_inds, np.newaxis, np.newaxis, np.newaxis]
+    )
     uv2.history = uv1.history
 
+    # make sure correction doesn't change data_array type
+    assert uv1.data_array.dtype == np.complex64
     assert "Divided out digital gains" in uv1.history
     assert uv1 == uv2
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set.")
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
-def test_remove_coarse_band():
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
+def test_remove_coarse_band(tmp_path):
     """Test coarse band removal."""
+    # generate a spoof file with 32 channels
+    cb_spoof = str(tmp_path / "cb_spoof_01_00.fits")
+    with fits.open(filelist[1]) as mini1:
+        mini1[1].data = np.repeat(mini1[1].data, 32, axis=0)
+        mini1.writeto(cb_spoof)
+
     uv1 = UVData()
-    uv1.read(filelist[0:2])
+    uv1.read([filelist[0], cb_spoof], data_array_dtype=np.complex64)
 
     uv2 = UVData()
-    uv2.read(filelist[0:2], remove_coarse_band=False)
+    uv2.read([filelist[0], cb_spoof], remove_coarse_band=False)
 
-    with open(DATA_PATH + "/mwa_config_data/MWA_rev_cb_10khz_doubles.txt", "r") as f:
-        cb = f.read().splitlines()
-    cb_array = np.array(cb).astype(np.float64)
-    cb_array = cb_array.reshape(32, 4)
+    with h5py.File(
+        DATA_PATH + "/mwa_config_data/MWA_rev_cb_10khz_doubles.h5", "r"
+    ) as f:
+        cb = f["coarse_band"][:]
+    cb_array = cb.reshape(32, 4)
     cb_array = np.average(cb_array, axis=1)
-    cb_array = cb_array[0]
+    uv2.data_array /= cb_array[:, np.newaxis]
 
-    uv2.data_array = uv2.data_array.swapaxes(2, 3)
-    uv2.data_array = uv2.data_array / cb_array
-    uv2.data_array = uv2.data_array.swapaxes(2, 3)
     uv2.history = uv1.history
 
-    assert "Divided out coarse channel bandpass" in uv1.history
+    # make sure correction doesn't change data_array type
+    assert uv1.data_array.dtype == np.complex64
+    assert "Divided out pfb coarse channel bandpass" in uv1.history
     assert uv1 == uv2
 
 
-def test_cotter_flags():
-    """Test using cotter flags"""
+def test_aoflagger_flags():
+    """Test using aoflagger flags"""
     uv = UVData()
     files = filelist[0:2]
     files.append(filelist[3])
     messages = [
-        "telescope_location is not set.",
         "some coarse channel files were not submitted",
         "coarse channel, start time, and end time flagging will default",
     ]
     with uvtest.check_warnings(UserWarning, messages):
-        uv.read_mwa_corr_fits(files, flag_init=False, remove_flagged_ants=False)
+        uv.read_mwa_corr_fits(
+            files, flag_init=False, remove_flagged_ants=False, correct_cable_len=False,
+        )
 
     with fits.open(filelist[3]) as aoflags:
         flags = aoflags[1].data.field("FLAGS")
@@ -707,14 +795,14 @@ def test_cotter_flags():
     assert np.all(uv.flag_array == flags)
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set.")
 @pytest.mark.filterwarnings(
     "ignore:coarse channels are not contiguous for this observation"
 )
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
 @pytest.mark.filterwarnings("ignore:coarse channel, start time, and end time flagging")
-def test_cotter_flags_multiple(tmp_path):
-    """Test cotter flags with multiple coarse bands"""
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
+def test_aoflagger_flags_multiple(tmp_path):
+    """Test aoflagger flags with multiple coarse bands"""
     mod_mini_6 = str(tmp_path / "mini_gpubox06_01.fits")
     with fits.open(filelist[2]) as mini6:
         mini6[1].header["time"] = 1447698337
@@ -737,9 +825,9 @@ def test_cotter_flags_multiple(tmp_path):
     assert np.all(uv.flag_array == flags)
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set.")
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
 @pytest.mark.filterwarnings("ignore:coarse channel, start time, and end time flagging")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
 def test_mismatch_flags():
     """Break by submitting flag and gpubox files from different coarse bands."""
     uv = UVData()
@@ -750,11 +838,11 @@ def test_mismatch_flags():
     assert str(cm.value).startswith("flag file coarse bands do not match")
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set. ")
 @pytest.mark.filterwarnings(
     "ignore:coarse channels are not contiguous for this observation"
 )
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
 def test_propagate_coarse_flags():
     """
     Test that the flag(without flag_int) and nsample arrays correctly reflect data.
@@ -764,8 +852,8 @@ def test_propagate_coarse_flags():
     assert np.all(uv.flag_array)
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set.")
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
 def test_start_flag(tmp_path):
     """Test the default value of start_flag."""
     uv1 = UVData()
@@ -777,8 +865,7 @@ def test_start_flag(tmp_path):
         edge_width=0,
         flag_dc_offset=False,
     )
-    good_ants = np.arange(128)
-    good_ants = np.delete(good_ants, [59, 114])
+    good_ants = np.delete(np.unique(uv1.ant_1_array), [59, 114])
     uv1.select(antenna_nums=good_ants)
     # start_time is after goodtime, so data for good antennas should be unflagged
     assert np.all(~uv1.flag_array)
@@ -799,8 +886,8 @@ def test_start_flag(tmp_path):
     assert np.all(uv2.flag_array)
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set.")
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
 def test_start_flag_goodtime_ppds():
     """Test that error is thrown using 'goodtime' with only ppds file."""
     uv = UVData()
@@ -809,8 +896,8 @@ def test_start_flag_goodtime_ppds():
     assert str(cm.value).startswith("To use start_flag='goodtime',")
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set.")
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
 def test_start_flag_bad_string():
     """Test that error is thrown if start_flag is given string other than 'goodtime'"""
     uv = UVData()
@@ -819,8 +906,8 @@ def test_start_flag_bad_string():
     assert str(cm.value).startswith("start_flag must be int or float or 'goodtime'")
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set.")
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
 def test_start_flag_int_time(tmp_path):
     """Test goodtime returning a start_flag smaller than integration time."""
     uv = UVData()
@@ -858,7 +945,6 @@ def test_input_output_mapping():
     return
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set. ")
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
 def test_van_vleck_int():
     """Test van vleck correction integral implementation."""
@@ -872,6 +958,7 @@ def test_van_vleck_int():
         remove_coarse_band=False,
         remove_dig_gains=False,
         remove_flagged_ants=False,
+        correct_cable_len=False,
     )
     # read in file corrected using integrate.quad with 1e-10 precision
     uv2 = UVData()
@@ -879,7 +966,6 @@ def test_van_vleck_int():
     assert np.allclose(uv1.data_array, uv2.data_array)
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set. ")
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
 def test_van_vleck_cheby():
     """Test van vleck correction chebyshev implementation."""
@@ -891,14 +977,14 @@ def test_van_vleck_cheby():
         cheby_approx=True,
         remove_coarse_band=False,
         remove_dig_gains=False,
+        correct_cable_len=False,
     )
     # read in file corrected using integrate.quad with 1e-10 precision
     uv2 = UVData()
     uv2.read(filelist[10])
 
     # select only good ants
-    good_ants = np.delete(np.arange(128), 76)
-    uv1.select(antenna_nums=good_ants)
+    good_ants = np.delete(np.unique(uv2.ant_1_array), 76)
     uv2.select(antenna_nums=good_ants)
 
     assert np.allclose(uv1.data_array, uv2.data_array)
@@ -914,8 +1000,8 @@ def test_van_vleck_interp(tmp_path):
         "values are being corrected with the van vleck integral",
     ]
     messages = messages * 10
-    messages.append("telescope_location is not set")
     messages.append("some coarse channel files were not submitted")
+    messages.append("Fixing auto-correlations to be be real-only,")
     uv = UVData()
     with uvtest.check_warnings(UserWarning, messages):
         uv.read(
@@ -925,11 +1011,12 @@ def test_van_vleck_interp(tmp_path):
             cheby_approx=True,
             remove_coarse_band=False,
             remove_dig_gains=False,
+            correct_cable_len=False,
         )
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set. ")
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
 def test_remove_flagged_ants(tmp_path):
     """Test remove_flagged_ants."""
     uv1 = UVData()
@@ -940,37 +1027,55 @@ def test_remove_flagged_ants(tmp_path):
     uv2.read(
         filelist[8:10], remove_flagged_ants=False,
     )
-    good_ants = np.delete(np.arange(128), 76)
+    good_ants = np.delete(np.unique(uv2.ant_1_array), 76)
+
     uv2.select(antenna_nums=good_ants)
 
     assert uv1 == uv2
 
 
-@pytest.mark.filterwarnings("ignore:telescope_location is not set. ")
 @pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+@pytest.mark.filterwarnings("ignore:.*values are being corrected with the van vleck")
+@pytest.mark.filterwarnings("ignore:cable length correction is now defaulted to True")
 def test_small_sigs(tmp_path):
-    """Test flag_small_sig_ants."""
+    """Test flag_small_auto_ants."""
     small_sigs = str(tmp_path / "small_sigs07_02.fits")
     with fits.open(filelist[8]) as mini:
         mini[1].data[0, 0] = 1000
         mini.writeto(small_sigs)
     uv1 = UVData()
     uv1.read(
-        [small_sigs, filelist[9]], correct_van_vleck=True, flag_small_sig_ants=True,
+        [small_sigs, filelist[9]], correct_van_vleck=True, flag_small_auto_ants=True,
     )
     messages = [
         "values are being corrected with the van vleck integral",
     ]
     messages = messages * 8
-    messages.append("telescope_location is not set")
     messages.append("some coarse channel files were not submitted")
+    messages.append("cable length correction is now defaulted to True")
     uv2 = UVData()
     with uvtest.check_warnings(UserWarning, messages):
         uv2.read(
             [small_sigs, filelist[9]],
             correct_van_vleck=True,
-            flag_small_sig_ants=False,
+            flag_small_auto_ants=False,
         )
 
     assert "flagged by the Van Vleck" in uv1.history
     assert uv2.Nants_data - uv1.Nants_data == 1
+
+
+@pytest.mark.filterwarnings("ignore:some coarse channel files were not submitted")
+def test_deprecated_keywords():
+    """Test warnings for deprecated keywords"""
+    uv = UVData()
+    messages = ["Use `use_aoflagger_flags`"]
+    messages.append("Use `flag_small_auto_ants`")
+    messages.append("some coarse channel files were not submitted")
+    messages.append("cable length correction is now defaulted to True")
+    with uvtest.check_warnings(
+        [DeprecationWarning, DeprecationWarning, UserWarning, UserWarning], messages,
+    ):
+        uv.read(
+            filelist[0:2], use_cotter_flags=False, flag_small_sig_ants=True,
+        )
